import com.aliyun.odps.spark.examples.myUtils.{FileUtils, ParamUtils}
import examples.utils.StatisticsUtil
import org.apache.spark.sql.SparkSession

import java.util
import scala.collection.JavaConverters._

object makedata_recsys_45_feature_cover_degree {
  def main(args: Array[String]): Unit = {
    val spark = SparkSession
      .builder()
      .appName(this.getClass.getName)
      .getOrCreate()
    val sc = spark.sparkContext

    val param = ParamUtils.parseArgs(args)
    val readPath = param.getOrElse("readPath", "/dw/recommend/model/41_recsys_origin_date/*")
    val featureNameFile = param.getOrElse("featureNameFile", "feature_name_20250218.txt")

    val resource = getClass.getClassLoader.getResource(featureNameFile)
    val fileContent = FileUtils.readFile(resource)

    // 解析特征名
    val featureNameSet = fileContent.split("\n")
      .map(_.trim)
      .filter(_.nonEmpty)
      .toSet
    val featureNameSet_br = sc.broadcast(featureNameSet)

    // 读取数据
    val data = sc.textFile(readPath)

    // 使用 Accumulator 代替 HashMap
    val allMapAcc = sc.collectionAccumulator[(String, Long)]("AllMap")
    val isShareMapAcc = sc.collectionAccumulator[(String, Long)]("IsShareMap")
    val isReturnNoSelfMapAcc = sc.collectionAccumulator[(String, Long)]("IsReturnNoSelfMap")

    data.foreach { line =>
      val allMap = new util.HashMap[String, java.lang.Long]()
      val isShareMap = new util.HashMap[String, java.lang.Long]()
      val isReturnNoSelfMap = new util.HashMap[String, java.lang.Long]()

      // 调用 Java 方法处理
      StatisticsUtil.featureCoverRate(line, featureNameSet_br.value.asJava, allMap, isShareMap, isReturnNoSelfMap)

      // 将结果累加到 Accumulator
      allMap.asScala.foreach { case (key, value) => allMapAcc.add((key, value)) }
      isShareMap.asScala.foreach { case (key, value) => isShareMapAcc.add((key, value)) }
      isReturnNoSelfMap.asScala.foreach { case (key, value) => isReturnNoSelfMapAcc.add((key, value)) }
    }

    // **转换 Accumulator 数据**
    val allMapSeq = allMapAcc.value.asScala.toSeq
    val isShareMapSeq = isShareMapAcc.value.asScala.toSeq
    val isReturnNoSelfMapSeq = isReturnNoSelfMapAcc.value.asScala.toSeq

    // **使用 groupBy 聚合**
    val finalAllMap = allMapSeq.groupBy(_._1).mapValues(_.map(_._2).sum).toMap
    val finalIsShareMap = isShareMapSeq.groupBy(_._1).mapValues(_.map(_._2).sum).toMap
    val finalIsReturnNoSelfMap = isReturnNoSelfMapSeq.groupBy(_._1).mapValues(_.map(_._2).sum).toMap

    // **输出最终结果**
    println("AllMap 结果:")
    finalAllMap.foreach { case (key, value) => println(s"$key -> $value") }

    println("IsShareMap 结果:")
    finalIsShareMap.foreach { case (key, value) => println(s"$key -> $value") }

    println("IsReturnNoSelfMap 结果:")
    finalIsReturnNoSelfMap.foreach { case (key, value) => println(s"$key -> $value") }
  }
}